/*
 *  Copyright (c) 1992-2012 Matisse Software Inc. All Rights Reserved.
 *
 *  This file (in both binary and source code form) is subject to the
 *  Supplemental License Terms governing use, modification and redistribution
 *  of Sample Code accompanying the Matisse(R) software.
 *
 */

#include "matisseCXX.h"

#include "company.h"

#include <iostream>
#include <sstream>
#include <iomanip>

using namespace std;

// namespaces containing Matisse classes
using namespace matisse;
using namespace matisse::reflect;
using namespace matisse::sql;

// namespaces containing the Schema classes
using namespace examples::cxx_examples::chaps_10_11;


// Same as newman1.cpp, except using SQL.
// 
// Usage
// See notes in newman1.cpp.

int main(int ac, char **av)
{
  int actualCreateCount = 0, actualTransCount = 0, actualAbortCount = 0;
  int newManagerCount = 50;   // default

  if (ac < 5)
  {
    std::cerr << "Usage: " << av[0]
              << " <host> <db> <asstFirstName> <asstLastName> [<count> <id>]"
              << std::endl;
    return -1;
  }

  try
  {
    MtDatabase db(av[1], av[2]);

    db.open();

    // Set the SQL CURRENT_NAMESPACE to 'dbNamespace' so there is
    // no need to use the full qualified names to acces the schema objects
    //db.setSqlCurrentNamespace("examples.cxx_examples.chaps_10_11");

    int useId = 999;

    std::string asstFirstName = av[3];
    std::string asstLastName = av[4];

    if (ac > 5)
      newManagerCount = ::atoi(av[5]);
    if (ac > 6)
      useId = ::atoi(av[6]);

    std::cout << "Creating " << newManagerCount
              << " managers with an assistant of "
              << asstFirstName << " " << asstLastName << std::endl;


    for (int i = 0; i < newManagerCount; i++)
    {
      try
      {
        actualTransCount++;
        db.startTransaction();

        MtStatement stmt(db);
        std::string selName = "sel1"; // selection name for
        // setting relationship

        // similar to non-SQL version, finding the employee
        // could be outside the loop, since it is static
        std::ostringstream sqs_emps;
        sqs_emps
          << "SELECT REF(e) FROM examples.cxx_examples.chaps_10_11.Employee e WHERE e.firstName = '"
          << asstFirstName << "' and e.lastName = '" << asstLastName
          << "' into " << selName;

        stmt.execute(sqs_emps.str());

#if 0
        // check
        MtResultSet res =  stmt.executeQuery("SELECT REF(s) FROM sel1 s");
        if (res.next())
        {
          Employee *e = dynamic_cast<Employee*>(res.getMtObject(1));
          std::cout << e->getFirstName() << " " << e->getLastName()
                    << std::endl;
        }
        else
        {
          std::cerr << "could not find " << asstFirstName << " "
                    << asstLastName << std::endl;
        }
#endif

        std::ostringstream sqs_man;

        // buld the insert command
        sqs_man << "INSERT INTO examples.cxx_examples.chaps_10_11.Manager(firstName, lastName, id, assistant) Values("
                << "'JIM', 'JACKSON" << std::setfill('0') << std::setw(6)
                << useId << "',"
                << useId << "," << selName << ");";

        int res = stmt.executeUpdate(sqs_man.str());
//                 std::cout << "Update: " << sqs_man.str() << std::endl
//                           << "Result : " << res << std::endl;

        if (res != 1)
        {
          // if insert command didn't have an exception,
          // result would be 1, so we should not reach this
          std::cerr << "Should not be here!" << std::endl;
          // but let continue
        }
        db.commit();
        actualCreateCount++;
      }
      catch (MtSQLException &mtsqle)
      {
        std::cerr << mtsqle << std::endl;
        if (db.isTransactionInProgress())
        {
          db.rollback();
        }
        actualAbortCount++;
      }
      catch (MtException &mte)
      {
        std::cerr << mte << std::endl;
        if (db.isTransactionInProgress())
        {
          db.rollback();
        }
        actualAbortCount++;
      }
      useId++;
    }

    db.close();

  }
  catch (MtException& mte)
  {
    std::cerr << mte << std::endl;
  }
  catch (std::exception& e)
  {
    std::cerr << "Standard exception: " << e.what() << std::endl;
  }
  catch (...)
  {
    std::cerr << "Unknown exception" << std::endl;
  }

  std::cout << "Summary: " << actualCreateCount
            << " managers created in " << actualTransCount
            << " transactions, with " << actualAbortCount << " rollbacks"
            << std::endl;

  return 0;
}

        
